use crate::prelude::*;
use serde::{Deserialize, Serialize};

#[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
pub struct InitialAmounts {
    pub min_funds: Amount,
    pub min_gift: Amount,
}

impl Default for InitialAmounts {
    fn default() -> Self {
        InitialAmounts {
            min_funds: 0,
            min_gift: 0,
        }
    }
}

impl InitialAmounts {
    pub fn gte(&self, other: &Self) -> bool {
        self.min_funds >= other.min_funds && self.min_gift >= other.min_gift
    }
}

#[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
pub struct Capacity {
    pub max_count: Natural,
    pub max_locked: Natural,
    pub max_single: Amount,
    pub max_total: Amount,
}

impl Default for Capacity {
    fn default() -> Self {
        Capacity {
            max_count: 30,
            max_locked: 30,
            max_single: E9,
            max_total: 5 * E9,
        }
    }
}

impl Capacity {
    pub fn gte(&self, other: &Self) -> bool {
        self.max_count >= other.max_count
            && self.max_locked >= other.max_locked
            && self.max_single >= other.max_single
            && self.max_total >= other.max_total
    }
}
