use crate::prelude::*;
use serde::{Deserialize, Serialize};

use super::common::{Capacity, InitialAmounts};

#[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
pub enum Config {
    Static(Terms),
}

impl Default for Config {
    fn default() -> Self {
        Config::Static(Terms::default())
    }
}

#[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
pub enum Res {
    Okay(Terms),
    Fail,
}

#[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
pub struct Terms {
    pub currencies: Vec<Currency>,
    pub initial_amounts: InitialAmounts,
    pub response_period: Milliseconds,
    pub capacity: Capacity,
    pub additional_terms: Markdown,
}

impl Default for Terms {
    fn default() -> Self {
        Terms {
            currencies: vec![ada_currency()],
            initial_amounts: InitialAmounts::default(),
            response_period: DAY,
            capacity: Capacity::default(),
            additional_terms: "".to_string(),
        }
    }
}

impl Terms {
    /// A channel adhering to `self` will necessarily adhere to other
    pub fn is_stricter(&self, other: &Self) -> bool {
        is_sublist(&self.currencies, &other.currencies)
            && self.initial_amounts.gte(&other.initial_amounts)
            && other.capacity.gte(&self.capacity)
    }
}

fn is_sublist<T>(a: &Vec<T>, b: &Vec<T>) -> bool
where
    T: Eq,
{
    a.iter().all(|x| b.iter().any(|y| x == y))
}

pub fn static_handle(terms: Terms) -> impl Fn() -> Res {
    move || Res::Okay(terms.clone())
}

pub fn handler(c: Config) -> impl Fn() -> Res {
    match c {
        Config::Static(terms) => static_handle(terms),
    }
}
