use std::{error::Error, time::Duration};

use libp2p::{
    mdns, noise,
    request_response::{self, ProtocolSupport},
    swarm::NetworkBehaviour,
    tcp, yamux, StreamProtocol, Swarm,
};
use libp2p_identity::ed25519::Keypair;

use crate::api::{Req, Res};

#[derive(NetworkBehaviour)]
pub struct MyBehaviour {
    pub mdns: mdns::tokio::Behaviour,
    pub req_res: request_response::cbor::Behaviour<Req, Res>,
}

pub fn mk_swarm(
    kp: &Keypair,
    protocol_support: ProtocolSupport,
) -> Result<Swarm<MyBehaviour>, Box<dyn Error>> {
    let swarm = libp2p::SwarmBuilder::with_existing_identity(kp.clone().into())
        .with_tokio()
        .with_tcp(
            tcp::Config::default(),
            noise::Config::new,
            yamux::Config::default,
        )?
        .with_behaviour(|key| {
            let mdns =
                mdns::tokio::Behaviour::new(mdns::Config::default(), key.public().to_peer_id())?;
            let protocol = [(StreamProtocol::new("/cll2p/1"), protocol_support)];
            let config = request_response::Config::default();
            let req_res = request_response::cbor::Behaviour::<Req, Res>::new(protocol, config);
            Ok(MyBehaviour { req_res, mdns })
        })?
        .with_swarm_config(|cfg| cfg.with_idle_connection_timeout(Duration::from_secs(u64::MAX)))
        .build();
    Ok(swarm)
}
